module eop_stuff
! Variables for storing EOP information 

private    !make the data in this module private 
public :: parse_spool_eop_data
public :: get_sessionID
public :: make_eop_magic_line 
public :: rmjd_beg, rmjd_end 
public :: write_eop_unit_lines
public :: Num_session 
public :: eop_sig_max

character*10 ldbname                       !Something like $20JAN04XA 
character*8 lsessionID                     !session ID from master file

integer Num_Session                         !Number of sessions we have read in.
double precision rmjd_beg, rmjd_end        !beginning, ending span of data  

integer iutc_prog(6)                       !iutc when program begins running. 
integer iutc_beg(6), iutc_end(6)           !iutc of first, last eop

Double precision   rmjd_eop
integer, parameter :: max_eop=10           !5 EOP and their rates.   In solve we do not estimate nutatin rates. 
integer              Num_EOP               !number of EOP read in. 
integer               ieop,jeop            !counters over eop. 
character*20 leop_type_spool(max_eop)            !
integer      ieop_val_off(max_eop)         !Where in the string do the eop values appear
double precision eop_val(max_eop)          !Eop values 
double precision eop_adj(max_eop) 
double precision eop_sig(max_eop)          !eop sigma (apriori)
double precision eop_sig_post(max_eop)     !eop sigma (post_fit).  Inflated by chi_sq. 
double precision cor_matrix(max_eop,max_eop) 


double precision eop_sig_max(max_eop)      !maximum allowed value of EOP. If greater than this not written out. 

! This is the preferred order of EOP:
! Xp,Yp, UT1, Nx, Ny, Xp_rate, Yp_rate, 
integer ieop_xref(max_eop)                 !Cross reference from order in spool file to prefered.
                                           ! Example. ieop(3)=5. This means UT1 is the 5th EOP value read in.
                                           !          ieop(1)=0. No Xp read in.                                                                         

integer NumObs                             !Number of obs for a session
double precision fit                       !Fit of session
Double precision TimeSpan                  !TimeSpan of session (hours) 

integer num_stat_in_session 

character*2 lnetwork(64)                   !maximum of 64 stations

! The following is based on the format of the spool file. If the format changes you need to change this.
! EOP lines are identified by having one of the following:
! Note that solve does not produce estimates of Nutation rates. 
data leop_type_spool/"X Wobble  0","Y Wobble  0","UT1-TAI   0", "Nutation DPSI","Nutation DEPS", &
               "X Wobble  1","Y Wobble  1","UT1-TAI   1", "Unused 1 ", "Unused 2"/         
! The EOP data starts this much after the identifier. 
data ieop_val_off/38,38,38,58,58,38,38,38,0,0/

contains
! 
! *************************************************************************
subroutine find_next_session(lu_spool, kcontrol_section_found,keof)
use common_variables 
implicit none
integer lu_spool
logical kcontrol_section_found
logical keof
! Read in the spool file a line at a time until you come to a new section or you hit the control file
logical kdebug 

kcontrol_section_found=.false.
keof=.false.
kdebug=.false.

ldum=" " 
do while(.true.)
   read(lu_spool,'(a)',end=900) ldum
   if(kdebug) then 
     writE(*,*) ldum
     read(*,*)  
   endif    
   
   if(ldum .eq. " ") cycle
!   write(*,*) trim(ldum) 
!  write(*,*) ldum(1:4) 
   if(ldum(1:6) .eq. "$SETUP") then
     kcontrol_section_found=.true.
     return
   endif 
   if(ldum(1:4) .eq. "1Run") then
      if(kdebug) writE(*,*) "New Session" 
      return      !start of new session.
   endif 
end do

900 continue
keof=.true.
return
end subroutine 
!************************************************************************
subroutine parse_spool_eop_data(lu_spool,lu_tmp,ierr)
! This routine reads in the eop part of the spool file. It exits when it gets to the control file. 
use master_stuff
use common_variables 
use time_stuff 
use nscodes_stuff
use leap_second_stuff
use header_stuff
implicit none
integer lu_spool
integer lu_tmp
integer ierr

! Functions
integer iwhere_in_string_list

! local variables
character*8 lstat_name_tmp 
integer istat
integer i 
logical kdebug
logical kcontrol_section_found
logical keof 
logical kfound_eop                         !true if we have found eop for some session. 

! get leap seconds
call read_leap_second_file(leap_second_file) 

! get list of stations and 2-letter codes. 
call read_nscodes(lnscodes_file) 

! Read in the masterfiles. This contains the correspondence between date and sessioncode, e.g. r12422
call read_masterfiles(lmaster_dir) 

! There are two options for reporting nutation. old style is Psi/Eps. New (post 2000) is Dx/Dy. 
if(lnutation_type .eq. "CIO-BASED") then 
  leop_type_spool(4)="Nutation Dx"
  leop_type_spool(5)="Nutation Dy"  
else 
  leop_type_spool(4)="Nutation DPSI"
  leop_type_spool(5)="Nutation DEPS"    
endif 
Num_Session=0                          !haven't processed any sessions yet. 

ierr=0
kdebug=.true. 
kdebug=.false. 
kfound_eop=.false.                     !Do we have an eop line to write?


! Make sure we are starting at the top of the spool file.
rewind(lu_spool)                !Need to do this because may have wandered into first session. 
! Start of main loop where we read in the spool file a line at a time. 
100 continue
! Start by spacing to next session, or until we hit control file. 
call find_next_session(lu_spool, kcontrol_section_found,keof)
if(kdebug) write(*,*) "kcontrol, keof: ", kcontrol_section_found, keof 

! If we have EOP info to write, do it.  
if(kfound_eop) then 
  num_session=num_session+1   
  call write_eop_line(lu_tmp)   
!  write(*,*) ldbname  
  kfound_eop=.false.
endif 

if(kcontrol_section_found) then 
  !Hit control file 
  call rmjd_2_utc(rmjd_beg, iutc_beg)     !convert data span to UTC and store it in common area 
  call rmjd_2_utc(rmjd_end, iutc_end)   
  close(lu_tmp)                           !Close out the file that contains the eop lines
  return  
endif   
if(keof) then
  write(*,*) "Hit end of spool file w/o finding control section"
  write(*,*) "Aborting"
  stop 
endif

! At this point we  have just read in a line that looks like "xRUN"
               
! Looking for a line like:
!> Data base $21JAN07XE Ver  4
do while (ldum(1:10) .ne. " Data base")   
  read(lu_spool,'(a)',end=900) ldum  
  if(kdebug) write(*,*) " line 154 >"//trim(ldum) 
end do
             
read(ldum(11:), *) ldbname               !read in the database name   
Num_EOP=0                                !Set to 0.  
ieop_xref=0                              !Set xref-array to 0.  0 values mean not found. 
lnetwork=" " 
num_stat_in_session=0
lsessionID=get_sessionID(ldbname)

if(kdebug) write(*,*) ldbname," ", lsessionID

! Get the release and version if we don't know it. 
if(lsolve_release .eq. "Unknown") then 
  do while(ldum(2:17) .ne. "Solve_release:")     
    read(lu_spool,'(a)',end=900) ldum 
  end do 
  lsolve_release=ldum(22:31)
  read(lu_spool,'(a)',end=900) ldum 
  lsolve_revision=ldum(22:31)
endif   

!look for lines that list the stations. There are many options but we will use the Elevation Cut line.
! This looks like:

! Fast_mode: B1B3D  Fast_cov: SEG                                              
! Elevation cut for HART15M  is  5.0
! Elevation cut for KOKEE    is  5.0
! Elevation cut for MEDICINA is  5.0
! Elevation cut for NYALE13S is  5.0
! Elevation cut for WETTZ13N is  5.0
! Elevation cut for WETTZELL is  5.0
!                                         
do while(ldum(1:18) .ne. " Elevation cut for")
 read(lu_spool,'(a)',end=900) ldum  
  if(kdebug) write(*,*) " line 181 >"//trim(ldum) 
end do  

!  write(*,*) "found"    
do while(ldum(1:18) .eq. " Elevation cut for")
  lstat_name_tmp=ldum(20:27)    
 !    write(*,*) ">"//lstat_name_tmp//"<"    
  ind=index(trim(lstat_name_tmp)," ")                     !this removes embedded spaces 
  if(ind .ne. 0) lstat_name_tmp(ind:ind)="_"                
  istat=iwhere_in_string_list(lstat_name,num_stat,lstat_name_tmp)      !find where it is in list of IVS stations
  if(istat .eq. 0) then  
    write(*,*) "Spool2eop: for database ",ldbname, " Station ",lstat_name_tmp, " not found"    
    cycle      
  endif  
  Num_stat_in_Session=num_stat_in_session+1
  lnetwork(num_stat_in_session)=lstat_code(istat)  
110  continue     
    read(lu_spool,'(a)') ldum   !prime for next station.
end do
 
do while(ldum(1:18) .ne. " Nominal duration:")
 read(lu_spool,'(a)',end=900) ldum  
 if(kdebug) write(*,*) " line 203 >"//trim(ldum) 
end do 

read(ldum(19:), *) TimeSpan
TimeSpan=TimeSpan/3600.              !convert from seconds to hours.  
 if(kdebug) write(*,*) " line 208: TimeSpan ", TimeSpan
 
!look for the line that has the fit and # of obs. this looks like:
!>   Delay     3731             23.713 ps              0.93       0.0000
do while(ldum(4:8) .ne. "Delay")
  read(lu_spool,'(a)',end=900) ldum  
end do 

read(ldum(13:37), *) NumObs, Fit
if(kdebug) then
  write(*,*) "NumObs Fit ", NumObs, Fit 
  write(*,*) "Hit any key to continue"
  read(*,*)
endif 
  
! Here is where we extract the EOP information.
! This is tricky because not all EOP components are estimated in every session. 
! So we loop here until we get to the end of this session or have found everything we need.

200 continue
read(lu_spool,'(a)',end=900) ldum 
if(kdebug) write(*,*) " line 229 >"//trim(ldum) 
if(ldum(1:29) .eq. " Corrected Reduced Chi-Square") goto 100    !End of current session, look for next session.
if(ldum(1:7) .eq. "EOP-MOD") goto 100                           !Are done processing sessions.  
if(ldum .eq. "  No constraints have been imposed") goto 100     !This is signal for intensives saying end of session.
 
! The next part of the file we are interested in is the EOP
! The EOP section in the SPOOL file looks like this:
! Note that > denotes the start of the line.  
!> 1143. X Wobble  0,  2022.04.22-06:27:00.000      73.0511 masec    -194.99 microasec      41.85 microasec      41.85 microasec
!> 1144. X Wobble  1,  2022.04.22-06:27:00.000       2.7112 mas/d     279.71 microas/d      82.65 microas/d      82.65 microas/d
!> 1145. Y Wobble  0,  2022.04.22-06:27:00.000     464.4365 masec      66.53 microasec      34.66 microasec      34.66 microasec
!> 1146. Y Wobble  1,  2022.04.22-06:27:00.000       1.1148 mas/d     115.15 microas/d      91.40 microas/d      91.40 microas/d
!> 1147. UT1-TAI   0,  2022.04.22-06:27:00.000  -37098.2194  msec       3.91 microsec        2.40 microsec        2.40 microsec
!> 1148. UT1-TAI   1,  2022.04.22-06:27:00.000       0.4195  ms/d      26.32 micros/d        4.25 micros/d        4.25 micros/d
!>                              Offset Error Ellipsoid Axis: Longitude =   79.9 Latitude =  24.7 Sigma    =      0.044 mas
!>                              Offset Error Ellipsoid Axis: Longitude = -169.4 Latitude =  37.5 Sigma    =      0.035 mas
!>                              Offset Error Ellipsoid Axis: Longitude =  -34.9 Latitude =  42.4 Sigma    =      0.034 mas
!>                                Rate Error Ellipsoid Axis: Longitude =  -32.8 Latitude =  -0.6 Sigma    =      0.097 mas
!>                                Rate Error Ellipsoid Axis: Longitude =   57.6 Latitude = -33.1 Sigma    =      0.083 mas
!>                                Rate Error Ellipsoid Axis: Longitude =   56.3 Latitude =  56.9 Sigma    =      0.053 mas
!>                               Earth Orientation   Rate Constraint Share 0.0
!> EOP epoch (TDT)  MJD: 59691.269122        NUT epoch (TDT)  MJD: 59691.269122
!> 1149. Nutation DPSI wrt   apriori model 2022.04.22-06:27:00.000    0.8930 mas       67.9 microasec       67.9 microasec       
!>       Nutation Dx   wrt   apriori model 2022.04.22-06:27:00.000    0.3552 mas       27.0 microasec       27.0 microasec       
!>       Nutation DPSI wrt Wahr 1980 model 2022.04.22-06:27:00.000 -106.0182 mas       67.9 microasec       67.9 microasec       
!> 1150. Nutation DEPS wrt   apriori model 2022.04.22-06:27:00.000   -0.1631 mas       24.1 microasec       24.1 microasec       
!>       Nutation Dy   wrt   apriori model 2022.04.22-06:27:00.000   -0.1631 mas       24.1 microasec       24.1 microasec       
!>       Nutation DEPS wrt Wahr 1980 model 2022.04.22-06:27:00.000   -9.5409 mas       24.1 microasec       24.1 microasec       

!> EOP Correlations:
!>  X Wob  Off  1.0000
!>  X Wob Rate -0.0766  1.0000
!>  Y Wob  Off  0.0839  0.0765  1.0000
!>  Y Wob Rate  0.0169 -0.2283 -0.1221  1.0000
!>UT1-TAI  Off -0.1884 -0.0146 -0.0059 -0.0070  1.0000
!>UT1-TAI Rate -0.0009  0.2922 -0.0299  0.1816 -0.0803  1.0000
!>         Psi -0.0321  0.2353  0.0251  0.3910  0.0092  0.0335  1.0000
!>         Eps  0.0327  0.2898  0.0609 -0.2972 -0.0317 -0.0199 -0.0347  1.0000
!>               X Wob   X Wob   Y Wob   Y Wob     UT1     UT1     Psi     Eps
!>                 Off    Rate     Off    Rate     Off    Rate

! Begin parsing this section. 

! Get the epoch    
if(ldum(1:22) .eq. " EOP epoch (TDT)  MJD:") then 
  read(ldum(24:35),*) rmjd_eop                        !read in the EOP
  rmjd_eop=rmjd_eop-32.184/86400.d0
  if(num_session .eq. 1) then
     rmjd_beg=rmjd_eop
     rmjd_end=rmjd_eop
  else
     rmjd_beg=min(rmjd_beg,rmjd_eop)
     rmjd_end=max(rmjd_end,rmjd_eop)
  endif   
  goto 200    
endif

! See if reading in an EOP value. Look for sub-strings like "X Wobble  0" etc.  
! If we found one of these then read in the eop_value, eop_sig, eop_sig_post. 
! The variable ieop_val_off is where the data starts w/r/t the EOP tag.   
do ieop=1,Max_EOP
  ind=index(ldum,trim(leop_type_spool(ieop)))                 !see if we found the identfier for EOP 
! Note that we use the value of ind below to read in the string.  
  if(ind .eq. 0) cycle  !Did not find string we were looking for.   
  kfound_eop=.true.                                           !Have found some estimated EOP values. 
! If we doing Equinox based, make sure we are using Wahr values. This is consistent with previous usage  
  if((ieop .eq. 4 .or. ieop .eq. 5) .and. lnutation_type .eq. "EQUINOX-BASED") then
    if(index(ldum,"Wahr") .eq. 0) cycle   !did not find Whar  
  endif 

  num_eop=num_eop+1   

  if(kdebug) then
     write(*,*) leop_type_spool(ieop) 
     writE(*,*) "line 293 ",trim(ldum)
     write(*,*) "Hit any key to continue ", num_eop 
     read(*,*) 
  endif 
 
! This bit of code replaces "/" by "x".  "/" can be interprted as EOL    
  i=index(ldum,"/")   
  do while(i .ne. 0)   
    ldum(i:i)="x"    
    i=index(ldum,"/")     
  end do     
  if(kdebug) write(*,*) " line 303 >"//trim(ldum) 
! Read in the line. If there is an error (most likely due to overflow) skip this session. Data is probably bad.   
  select case(ieop)
    case(1,2,3,6,7,8)    !For most eop read in total, adjustment, sigma,sigma
      read(ldum(ind+ieop_val_off(ieop):),*,err=100) eop_val(num_eop), ltmp1, eop_adj(num_eop), ltmp2, eop_sig(num_eop), &
&                                            ltmp3, eop_sig_post(num_eop)    
!        write(*,*) eop_val(num_eop), ltmp1, eop_adj(num_eop),  ltmp2, eop_sig(num_eop), ltmp3, eop_sig_post(num_eop) 
    case(4,5,9,10)       !For nutation read in total, sigma,sigma 
      read(ldum(ind+ieop_val_off(ieop):),*,err=100) eop_val(num_eop), ltmp1, eop_sig(num_eop), ltmp2, eop_sig_post(num_eop)    
  end select 
 
  ieop_xref(ieop)=num_eop                             !Set up xreference array. 
  if(eop_sig_max(ieop) .eq. 0) cycle                  !Don't check sigma value
  if(eop_sig(num_eop) .gt. eop_sig_max(ieop)) then
     if(kdebug) then
       write(*,*) "Throwing away "//trim(ldum)    
       write(*,*) "Because "//trim(leop_type_spool(ieop))//" sigma ", eop_sig(num_eop), "> ",eop_sig_max(ieop)  
       kfound_eop=.false. 
       write(*,*) "Hit any key to continue" 
       read(*,*)       
     endif 
     goto 100              !EOP is too large. Don't write out this line. Instead go to next session      
  endif 
  goto 200 ! get next line 
end do 

! Did not find any EOP values. See if we are at the correlation matrix. 
! If so read the matrix in.  
! This is a little complicated because the number of elements depends on the number estimated.
if(ldum(1:18) .eq. " EOP Correlations:") then 
  if(kdebug) write(*,*) "At correlations ",Num_eop 
! Read in a triangular matrix of correlations.
! First line has 1 value, 2nd 2 values, 3rd, 3values etc. 
  do ieop=1,Num_EOP 
    read(lu_spool,'(a)') ldum   
    read(ldum(13:),*) cor_matrix(ieop,1:ieop)          
  end do 
! Above read in lower diagonal. Copy over to upper diagonal.   
  do ieop=1,Num_EOP
    do jeop=ieop+1,Num_EOP
       cor_matrix(ieop,jeop)=cor_matrix(jeop,ieop)
    end do
  end do        
  goto 100    ! Have gotten all the EOP info we can 
endif 

goto 200     !Did not find what we were looking for, read next line. 


900 continue
ierr=-1
write(*,*) "Reached end of spool file w/o encountering control file"
return
end subroutine

!**************************************************************************************
subroutine write_eop_unit_lines(lu_out,lnutation_type)
implicit none
integer lu_out 
character*(*) lnutation_type 
character*450 lwhat
character*450 lunit

!if(.true.) then 
if(lnutation_type .eq. "CIO-BASED") then 
lwhat =&
& "#   epoch        xPol         yPol       dUT1         dX         dY        sig_xP     sig_yP     sig_UT     sig_dX  " & 
&//"   sig_dY     wRMS     cor_xPyP   cor_xPUT  cor_yPUT   cor_dXdY    Nobs SessID      Span      xPolR        yPolR" & 
&//"        LOD             dXR          dYR     sig_xPR      sig_yPR    sig_LOD         sig_dXR     sig_dYR   Network "                                                                                              
else 
lwhat =&
& "#   epoch        xPol         yPol       dUT1         dPsi       dEps      sig_xP     sig_yP     sig_UT     sig_dPsi" & 
&//"   sig_dEps   wRMS     cor_xPyP   cor_xPUT  cor_yPUT   cor_dPdR    Nobs SessID      Span      xPolR        yPolR" & 
&//"        LOD             dPR          dER     sig_xPR      sig_yPR    sig_LOD         sig_dPR     sig_dER   Network "              
endif 

lwhat(437:444)="Comment"

lunit= &
& "#   [MJD]        [as]         [as]       [s]          [mas]      [mas]      [as]       [as]       [s]  " & 
&//"      [mas]      [mas]     [ps]       [-]        [-]         [-]       [-]       [-]   [-]        [h]      " & 
&//"[as/d]       [as/d]       [s]           [mas/d]     [mas/d]    [as/d]       [as/d]       [s]     "& 
& //"      [mas/d]     [mas/d]     [-] "   
lunit(437:439)="[-]"          

write(lu_out,'(a)') lwhat
write(lu_out,'(a)') lunit 

end subroutine

!********************************************************************
subroutine write_value(lu_out, kfound, rvalue, lformat1,lformat2)
implicit none
! If kfound is true,   write out the value using lformat(1).
!    kfound is fallse, write out lformat(2)    
integer lu_out
!character*(*) lu_out 
logical kfound
double precision rvalue
character*(*) lformat1          !format for number
character*(*)  lformat2          !otherwise a string


if(kfound) then 
   write(lu_out,lformat1) rvalue
else
   write(lu_out,lformat2)
endif
return 
end subroutine 

!********************************************************************************
subroutine write_eop_line(lu_eop)
use leap_second_stuff 
implicit none
integer lu_eop
!character*512 lu_eop 
!There are 31 fields. We write out each field one at a time. If we have no value for the field we output "NA".

integer i                                          !counter                       !
character*20 leop_format(max_eop)
double precision eop_scale(max_eop)
double precision eop_sig_scale(max_eop)
character*20 leop_NA(max_eop), leop_NA2(max_eop)

! First format is if we found the value, second is if we didn't find it. 
character*20, parameter :: lf8p4 ='(f8.4,  " ",$)' 
character*20, parameter :: lf8p5 ='(f8.5,  " ",$)' 
character*20, parameter :: lf9p4 ='(f9.4,  " ",$)' 
character*20, parameter :: lf9p5 ='(f9.5,  " ",$)' 
character*20, parameter :: lf10p5='(f10.5, " ",$)' 
character*20, parameter :: lf10p6='(f10.6, " ",$)' 
character*20, parameter :: lf10p7='(f10.7, " ",$)' 
character*20, parameter :: lf11p5='(f10.5, " ",$)' 
character*20 ,parameter :: lf12p7='(f12.7, " ",$)'   
character*20 ,parameter :: lf12p8='(f12.8, " ",$)'        
character*20 ,parameter :: lf12p9='(f12.9, " ",$)'         

! Following formats have NA embedded.  NA is always at end followed by a space.   
! The 'NA' lines up with the last digit of the number for non-NA                              
!                                    123456789x123456789x 
character*20, parameter :: lf8NA ='("      NA ",$)'        
character*20, parameter :: lf9NA ='("       NA ",$)'   
character*20, parameter :: lf10NA='("        NA ",$)'     
character*20, parameter :: lf11NA='("         NA ",$)'                      
character*20, parameter :: lf12NA='("          NA ",$)'    !
character*20, parameter :: lf14NA='("            NA ",$)'  !
character*20 lform1
character*20 lform2

integer iptr 
double precision temp
logical kfound              !True if we found an EOP value or correlation 
integer i1,i2               !temporary variables 
integer ival_or_sig 


! This converts from units in the spool file to units we write.      
! Notice minus sign for 8th component. This converts from UT1 rate to LOD
data eop_scale        /1.d-3, 1.d-3,  1.d-3, 1.d0,  1.d0,  1.d-3,1.d-3, -1.d-3, 1.d0, 1.d0/
data eop_sig_scale    /1.d-6, 1.d-6,  1.d-6, 1.d-3, 1.d-3, 1.d-6,1.d-6, 1.d-6, 1.d0, 1.d0/ 

!Below is for debugging
!write(lu_eop,'(i5," ", $)') num_session 


!Field 1.      EPOCH
write(lu_eop,'(f12.6,"  ",$)') rmjd_eop

! Now we output the values or the sigmas. 
! First we output values, then the sigmas.
! Note that the precision for the values is the same as the sigmas

! First time we go through outer loop output values.   Fileds 2-6
! Second time we go through we output sigmas           Fields 7-11
do ival_or_sig=1,2 
  do ieop=1,5
    if(ieop_xref(ieop) .ne. 0) then
      kfound=.true.
      iptr=ieop_xref(ieop) 
      if(ival_or_sig .eq. 1) then 
        temp=eop_val(iptr)*eop_scale(ieop)                      !pick out the appropriate value and rescale. 
        if(ieop .eq. 3) then 
          temp=temp+ileap_Second_now(rmjd_eop)
        endif 
      else
        temp=eop_sig(iptr)*eop_sig_scale(ieop)                  !pick out the approrpiate value and rescale. 
      endif 
    else
      kfound=.false.
    endif 
    select case(ieop)
      case(1,2)          !X&Y
        lform1=lf10p7
        lform2=lf10NA      
      case(3)
        lform1=lf12p8
        lform2=lf12NA
      case(4,5)   
        lform1=lf10p5
        lform2=lf10NA
    end select   
    call write_value(lu_eop,kfound,temp,lform1,lform2)    !write out the value
  end do   
end do 

! Field 12.  Fit of session
write(lu_eop,'(f7.3,"  ",$)') fit 

! Field 13-16 are elements of the correlation matrix.
do i=1, 4
  select case(i)
    case(1)           !output Cor_xP_yP
      i1=1
      i2=2
    case(2)           !output Cor_xp_UT1
      i1=1
      i2=3
    case(3)           !output Cor_yP_Ut1
      i1=2
      i2=3
    case(4)           !output Cor_dNx_dNy
      i1=4
      i2=5 
  end select  
  kfound = ieop_xref(i1) .ne. 0 .and. ieop_xref(i2) .ne. 0
  if(kfound) temp=cor_matrix(ieop_xref(i1),ieop_xref(i2)) 
   call write_value(lu_eop,kfound, temp, lf10p6, lf10NA)
end do     

! Field 17.  NumObs
write(lu_eop, '(i6,"  ",$)') NumObs

! Field 18.  
write(lu_eop, '(a8,"  ",$)') lsessionID

! Field 19
write(lu_eop, '(f6.3,"  ",$)') TimeSpan

! Now we output the EOP rates and sigmas.     Fields 20-24
! First time we output the rates.             Fields 25-29 

do ival_or_sig=1,2 
  do ieop=6,10
    if(ieop_xref(ieop) .ne. 0) then
      kfound=.true.
      iptr=ieop_xref(ieop) 
      if(ival_or_sig .eq. 1) then 
        temp=eop_val(iptr)*eop_scale(ieop)                !pick out the approrpiate value and rescale. 
      else
        temp=eop_sig(iptr)*eop_sig_scale(ieop)                  !pick out the approrpiate value and rescale. 
      endif 
    else
      kfound=.false.
    endif 
    select case(ieop)
      case(6,7)          !X&Y
        lform1=lf12p9
        lform2=lf12NA      
      case(8)
        lform1=lf12p9
        lform2=lf12NA
      case(9,10)   
        lform1=lf11p5
        lform2=lf11NA
    end select   
    call write_value(lu_eop,kfound,temp,lform1,lform2)    !write out the value
  end do   
end do 

writE(lu_eop,'("  ",$)')   !output a couple of spaces. This makes the line more readable 
! Field 30.  Network
do i=1,num_stat_in_session-1
  write(lu_eop, '(a2,"-",$)') lnetwork(i)
end do   
write(lu_eop,'(a2,$)') lnetwork(num_stat_in_session) 

! This just makes the database line up nicely
do i=num_stat_in_session,32
  write(lu_eop,'("   ",$)') 
end do 


write(lu_eop, '( "!  ",a)') ldbname 
  
  
  
  
!write(lu_eop,'(a)') 
! Field 31.  Comment field.
!write(lu_eop,'("! Done ")')
return 
end subroutine write_eop_line

end module eop_stuff
