module header_stuff 

! Below come from the control file.This will override anything in the spoolfile.
! May want to use this to replace  a filename with a description
character*80 lcrf_apriori_override
character*89 ltrf_apriori_override
characteR*80 leop_apriori_override
character*80 ldescription_override
character*3  lagency_override
character*80 leop_subdaily_override 
character*80 lcontact
character*80 lanalysis_center 

! Variables and subroutines dealing with the header.
character*3  lagency   !Institution from spool file. 
character*80 ldescription     !Description from spool file
character*10 lsolve_release, lsolve_revision 
character*30 lcontrol_file    !Name of the control file.
character*20 lsolve_run_time  !when the solution was run. 

! Aprirori from spool file. These are file names.
character*80 lcrf_apriori
character*80 lapriori_position
characteR*80 lapriori_velocity
character*80 leop_apriori
character*80 lPSD_apriori 
character*80 lhf_eop_apriori
character*20 leop_subdaily    !Desai, something else. 

! Note. ltrf_apriori is either taken from control file, or constructed from position and velocity mod files. 
character*200 ltrf_apriori

! 
logical keop_est(10)         !X,Y,UT1, Nx, Ny, followed by rates. 
double precision eop_constraints(10)     !constraints.  0 means no constraints. 

! below is for outputing in the eop file. 
character*10 leop_type(10)
character*8 leop_units(10) 
data leop_type/"XPOL","YPOL","DUT1", "DPSI","DEPS", &
     &          "XPOL_DER_1","YPOL_DER_1","LOD","DPSI_DER_1","DEPS_DER_1"/     

data leop_units/"as",   "as",    "s","mas",   "mas", &
     &         "as/day","as/day","s","mas/day","mas/day"/


contains
! ********************************************************************************************************
subroutine parse_spool_header(lu_spool,ierr)
use common_variables
implicit none
integer lu_spool
integer ierr 

integer num_lines 
integer, parameter :: max_lines=20     !once we get to the section only parse 20 lines max. 
logical kdebug

! We parse the spool file until we find a section that looks like below, and extract information that we need.

! 
! ---------------------------------------------------------------------------
! Analysis center: GSF -- NASA Goddard Space Flight Center
! Analyst:           (   )
! Machine:         gs61av-vlbidev01  x86_64  Linux 5.4.0-96-generic
! Executables:     /sgpvlbi/projects/jgipson/csolve_dev/bin/
! Solve initials:  JG
! Solution ID:     2021c csolve final global quarterly solution
! Spool format:    Revision date 2003.08.20
! Parameter timetags are in UTC.
! Local time:      2022.05.06-12:38:12
! ---------------------------------------------------------------------------
! Listing_Options:  CRES_EMULATION NO  BASELINES NO  MINIMUM NO
! Listing_Options:  MAPPED_EOP_OUTPUT NO  SEG_OUTPUT NO  APRIORI_ZENDEL NO
! Listing_Options:  NRD_TABLE YES  CHI_TABLE NO  SRC_STAT LONG  SEG_STYLE POST2005
! Solve_release:    2021.04.07
! Solve_revision:   2022-01-08

! The top of the control file should look like this:  (Data in file starts immediately after ">"
!>ENTER jg 2021c_v3_mcg_bak.cnt                     
!>Run done on gs61av-v with executables from /sgpvlbi/projects/jgipson/csolve_dev/bin/
!>Run started at 2022.05.06-12:37:00 local time

! Want to extract the control_file_name and the runtime. 
kdebug = .false.  
lsolve_release="Unknown"
lsolve_revision="Unknown"

read(lu_spool, *,end=200) ltmp1,ltmp2,lcontrol_file
if(ltmp1 .ne. "ENTER") then
  write(*,*) "Expected first line to start with 'ENTER xx control_file_name'" 
  write(*,*) "Setting control_file_name to 'unknown'"
  lcontrol_file='unknown'
  stop
endif 

! skip a line
read(lu_spool,'(a)',end=200) ldum 
read(lu_spool,'(a)',end=200) ldum
lsolve_run_time=ldum(16:34)


num_lines=0
ierr=0
!Space until we get to the appropriate section. 

100 continue
read(lu_spool,'(a)',end=200) ldum
if(kdebug) write(*,*) trim(ldum) 
ind=index(ldum,"Analysis center:")
if(ind .eq. 0) goto 100 
goto 111
 

110 continue 
read(lu_spool,'(a)', end=200) ldum

111 continue 
num_lines=num_lines+1
if(num_lines .gt.max_lines) goto 200
select case(ldum(2:17))
  case("Analysis center:")      
     read(ldum(19:), '(a)') lanalysis_center  
     read(lanalysis_center,*) lagency       
  case("Solution ID:") 
     ldescription=ldum(19:)
  case("Solve_release:") 
     lsolve_release=ldum(22:31)
  case("Solve_revision:")
     lsolve_revision=ldum(22:31)     
     return 
  case default    
end select
goto 110 

200 continue
! Come here if we don't get all of the information.
ierr=-1
return 
end subroutine parse_spool_header
!************************************************************************
subroutine parse_spool_control_section(lu_spool,ierr)
! Process the control sectionk, extracting what we need. 
use common_variables 
implicit none
integer lu_spool 
integer ierr

ierr=0

! Note: On entry ldum has latest line read from the spool file. It should be $CONTROL
! This loop processes various parts of the control file. 
! On entry we have just read the first line of some section (beginning with the section identifier)
do while(.true.)
  read(ldum, *,end=900) ltoken(1)        !this should be some section ($CONTROL, $FLAGS, ...) or a comment
  if(ldum(1:1) .eq. "*")  cycle
  ierr=0 
  ldum=" " 
! See if we are a processing a known section.  
  select case(ltoken(1)) 
    case("$FLAGS")
      call parse_flags_section(lu_spool,ierr)
    case("$MAPPING")    
      call parse_mapping_section(lu_spool,ierr)
    case("$CONSTRAINTS") 
      call parse_constraints_section(lu_spool,ierr)
    case default  
! Read until we come to the next section.
     do while(ldum(1:1) .ne. "$")
       read(lu_spool,'(a)',end=900) ldum
    end do     
  end select 
  if(ierr .ne. 0) goto 900            !some error message--probably an EOF. We are done.
end do      
   
900 continue
close(lu_spool) 
return
end subroutine
! ************************************************************************************
subroutine parse_flags_section(lu_spool,ierr)
use common_variables
implicit none 
integer lu_spool
integer ierr

! Default is set all EOP to not estimated. 
keop_est=.false. 
! Extract information from flags section that we need.  Exit when we get to the next section.
do while(.true.) 
  read(lu_spool,'(a)',end=900) ldum
  if(ldum(1:1) .eq. "*") cycle         !ignore comments
  if(ldum(1:1) .eq. "$") return       !Done with this section 
  call splitntokens(ldum,ltoken,Max_Token,Num_Token)      
  select case(ltoken(1))                 
    case("UT1/PM")  
      if(ltoken(2) .eq. "POLYNOMIAL") then
! Line looks someting like:
!     UT1/PM           POLYNOMIAL OFFSET XYU RATE XYU 2ND_ORDER --- MIDDLE  
        keop_est(1)=ltoken(4)(1:1) .eq. "X" 
        keop_est(2)=ltoken(4)(2:2) .eq. "Y" 
        keop_est(3)=ltoken(4)(3:3) .eq. "U" 
! look at rates
        keop_est(6)=ltoken(6)(1:1) .eq. "X" 
        keop_est(7)=ltoken(6)(2:2) .eq. "Y" 
        keop_est(8)=ltoken(6)(3:3) .eq. "U"            
      endif 
    case("NUTATION")
       if(ltoken(2) .eq. "OFFSET") then
          keop_est(4)=.true.
          keop_est(5)=.true. 
       endif     
    case default   
   end select  
end do 

900 continue
ierr=-1
return
end subroutine 
! ************************************************************************************
subroutine parse_mapping_section(lu_spool,ierr)
use common_variables
implicit none 
integer lu_spool
integer ierr

lcrf_apriori="NONE "
lapriori_position="NONE "
lapriori_velocity="NONE "
leop_apriori="NONE"
lpsd_apriori="NONE"
lhf_eop_apriori="NONE" 

! Extract informatin from Mapping section that we need.  Exit when we get to the next section.
do while(.true.) 
  read(lu_spool,'(a)',end=900) ldum
  if(ldum(1:1) .eq. "*") cycle         !ignore comments
  if(ldum(1:1) .eq. "$") return         !Done with this section 
  call splitntokens(ldum,ltoken,Max_Token,Num_Token)       
  select case(ltoken(1))                 
    case("SOURCES")
      lcrf_apriori   =ltoken(2)
    case("STATIONS")
      lapriori_position =ltoken(2)
    case("VELOCITIES")
      lapriori_velocity =ltoken(2)
    case("EARTH_ORIENTATION") 
      leop_apriori      =ltoken(2)
    case("EARTHQUAKE") 
      lPSD_apriori      =ltoken(2) 
    case("HFEOP_XYU")
      lhf_eop_apriori    =ltoken(2)            
    case default   
   end select  
end do 

900 continue
ierr=-1
return
end subroutine 

! *******************************************************************************
subroutine parse_constraints_section(lu_spool,ierr)
use common_variables
implicit none 
integer lu_spool
integer ierr
! local
integer i
double precision temp 

eop_constraints=0.d0            !Default is no constraints. 
do while(.true.) 
  reaD(lu_spool,'(A)',end=900) ldum 
  if(ldum(1:1) .eq. "*") cycle    
  if(ldum(1:1) .eq. "$") return
  call splitntokens(ldum,ltoken,Max_Token,Num_Token)     
  select case(ltoken(1))   
    case("EARTH_ORIENTATION") 
! looking for a line that looks like     
!  EARTH_ORIENTATION  YES   SIGMA  45.0 45.0  3.0 RATES SIGMA  45.0 45.0  3.0    
! Note that the units are in mas and ms.   
! Read in the constraints and rescale. 
    if(ltoken(2) .eq. "YES") then
       do i=1,3 
          read(ltoken(3+i),*) temp
          eop_constraints(i)=temp*1.d-3
          read(ltoken(8+i),*) temp
          eop_constraints(i+5)=temp*1.d-3
       end do 
     endif 
   case("NUTATION")
     if(ltoken(2) .eq. "YES") then
! have a line that lookes like
!  NUTATION YES SIGMA  xxx yyy
        read(ltoken(4),*) eop_constraints(4)
        read(ltoken(5),*) eop_constraints(5)   
     endif
  end select      
end do 

! Come here on error.
900  continue
ierr=-1
end subroutine


end module header_stuff
