module leap_second_stuff
! Stuff dealing with leapseconds
private
public :: read_leap_second_file
public :: ileap_second_now

integer, parameter :: max_leap_second=100    !maximum number of leap seconds
integer  num_leap_second
! We are going to read in the leap seconds.  

double precision rmjd_leap(max_leap_second)
integer ileap_second(max_leap_second)

contains
!**********************************
subroutine read_leap_second_file(leap_second_file)
implicit none
! Read in leap second file 
character*(*) leap_second_file
integer, parameter :: lu_in=15 
character*80 ldum 
integer iyear
double precision fjday     !full julian day . 
integer i

! local
logical kexist

inquire(file=trim(leap_second_file),exist=kexist)
if(.not.kexist) then
  write(*,*) "Leap second file does not exist: "//trim(leap_second_file)
  stop
endif

open(lu_in, file=trim(leap_second_file))

! Read in the file. The lines look like:
! 1972 JAN  1 =JD 2441317.5  TAI-UTC=  10.0       S + (MJD - 41317.) X 0.0      S
! 1972 JUL  1 =JD 2441499.5  TAI-UTC=  11.0       S + (MJD - 41317.) X 0.0      S
! 1973 JAN  1 =JD 2441683.5  TAI-UTC=  12.0       S + (MJD - 41317.) X 0.0      S
! 1974 JAN  1 =JD 2442048.5  TAI-UTC=  13.0       S + (MJD - 41317.) X 0.0      S 

num_leap_second=0
do while(.true.) 
  read(lu_in, '(a)', end=200) ldum  
  read(ldum,*) iyear 
! Since the first VLBI data we are interested in starts in 1979, discard data before then.   
  if(iyear .lt. 1972) cycle      
  num_leap_second=num_leap_Second+1 
  if(num_leap_second .gt. max_leap_second) then
     write(*,*) "Out of space to store leap second data"
     stop
  endif 
  read(ldum(18:27), *, err=300,end=300) fjday
  rmjd_leap(num_leap_second)=fjday-  2400000.5d0
  read(ldum(39:40),*) ileap_second(num_leap_second)
! ignore repeated values...  
  if(num_leap_second .ge. 2) then
    if(ileap_second(num_leap_second) .eq. ileap_second(num_leap_second-1)) then
      num_leap_second=num_leap_second-1
    endif
  endif 
end do 
200 continue

if(.false.) then 
  do i=1,num_leap_second
  write(*,*) rmjd_leap(i), " ", ileap_second(i) 
  end do
endif 

close(lu_in)
return 

300 continue
write(*,*) "Error parsing leap second line "
write(*,*) trim(ldum)
stop
return
end subroutine 
!*****************************************************
integer function ileap_second_now(rmjday)
implicit none 
! Given the modified julian day return associated leap second
double precision rmjday
integer ibot, itop, itest 
integer iter 

if(rmjday .lt. rmjd_leap(1)) then 
  write(*,*) "Can only return leap seconds after ",rmjd_leap(1)
  write(*,*) "You asked for ", rmjday
  stop
else if(rmjday .ge. rmjd_leap(num_leap_second)) then
  ileap_second_now=ileap_second(num_leap_second)
  return 
endif

! Know the input epoch is AFTER the first epoch and less than the last epoch. 

! Find first time that rmjd_leap(i) <= rmjday < rmjd_leap(i+1)
! could just search throught the table, but can speed up a bit by doing bisection


ibot=1
itop=num_leap_second
itest=(itop+ibot)/2
iter = 0

do while(iter .lt. 8)    !this can handle a table of upto 128 
  iter = iter+1
  itest=(itop+ibot)/2
!   write(*,*) rmjd_leap(itest), rmjday, rmjd_leap(itest+1) 
   if(rmjday .ge. rmjd_leap(itest)) then
      if(rmjday .le. rmjd_leap(itest+1)) then
         ileap_second_now=ileap_second(itest) 
!         write(*,*) "leap second ", ileap_second_now 
!         pause 
         return
      else 
        ibot=itest
      endif
   else
     itop=itest
   endif  
end do 
write(*,*) "Failed to find leap second after ", iter, " iterations "
write(*,*) "MJDAY ", rmjday

write(*,*) "trying again" 

! Now redo with debugging on.
ibot=1
itop=num_leap_second
itest=(itop+ibot)/2
iter = 0

do while(iter .lt. 6)
  iter = iter+1
  itest=(itop+ibot)/2
   writE(*,*) ibot, itop, itest 
   write(*,*) rmjd_leap(itest), rmjday, rmjd_leap(itest+1) 
   if(rmjday .ge. rmjd_leap(itest)) then
      if(rmjday .le. rmjd_leap(itest+1)) then
         ileap_second_now=ileap_second(itest) 
         write(*,*) "leap second ", ileap_second_now, " type any key to proceed" 
         read(*,*) 
         return
      else 
        ibot=itest
      endif
   else
     itop=itest
   endif  
end do 

if(.true.) then 
  do iter=1,num_leap_second
  write(*,*) iter, rmjd_leap(iter), " ", ileap_second(iter) 
  end do
endif 


stop


end function
end module 
















