module master_stuff
! data and subroutines dealing with the master file. 

private    !make the data in this module private 
public :: Read_Masterfiles
public :: get_sessionID

!character*12, parameter :: lmaster_dir="Data/"
integer, parameter :: lu_master=33

type session
  character*8 lSessionID  
  integer     idoy 
  integer*4   mjday 
  character*2 ldbc
end type 
  
integer, parameter :: max_session=100000
integer,private :: num_session
type(session) :: session_list(max_session)

contains
!*******************************************************
logical function kGE_session(session_one,session_two)
implicit none
type(session) :: session_one, session_two
! Return true of session_one => session_two

kge_session=.false. 
if(session_one%mjday .gt. session_two%mjday) then
  kge_session=.true. 
else if(session_one%mjday .lt. session_two%mjday) then
  kge_session=.false.
else  
  kge_session=session_one%ldbc .ge. session_two%ldbc 
endif 
!write(*,*)  session_one
!write(*,*)  session_two
!write(*,*)  Kge_session
!pause 
           
return
end function 
!*******************************************************
subroutine Read_Masterfiles(lmaster_dir) 
use time_stuff 
implicit none
! Subroutine to read in the masterfiles
character*(*) lmaster_dir
character*128 lmasterfile

integer iyear2
integer iyear 
character*10 lsuffix
integer iyear_beg,iyear_end
character*150 ldum 

integer itype                   !Counter over master file types. 
logical kexist 
logical       kopen(3)          !Can have upto 3 master files for a given year. 
type(session) session_tmp(3)    !Can have one session from each master file. 
integer lu_master(3)    
integer num_bar(3)
integer, parameter ::max_bar=20
integer ibar_vec(3,max_bar) 

integer i,j 
logical kdebug 
integer time_values(8)
integer iyear_now
integer mjday_off    !mjday at start of year
integer idoy        
    
kdebug=.false. 

lu_master(1)=21
lu_master(2)=22
lu_master(3)=23
num_session=0 

call date_and_time(VALUES=time_values)
iyear_now=time_values(1)

! General strategy.
!1. For each year, open all the master files.
!2. Read in a session from each master file. 
!3. Find the session which is earliest. 
!4. Insert that into the list and read another session from that file.
!5. If no more close that file.
!6. If all files closed, then we are done.
!7. Goto 3. 
!2. After each year, sort the sessions by time-tag, two-letter code.  

do iyear=1979, iyear_now+1      !look ahead to following year
   mjday_off=ymd_2_mjday(iyear,1,1) -1   !Mjd for Dec31 previous year 

   if(iyear .lt. 2000) then
      iyear2=iyear-1900
    else
      iyear2=iyear-2000
   endif
   do itype=1,3 
     kopen(itype)=.false.               !Default is this type of master file is not open. 
     select case(itype)
       case(1)     
         lsuffix=".txt"
       case(2)
         if(iyear .lt. 1992) cycle       !No intensive master file before 1992     
         lsuffix="-int.txt"
       case(3)
         if(iyear .lt. 2015 .or. iyear .gt. 2019) cycle       
         lsuffix="-vgos.txt"
     end select      
     write(lmasterfile,'(a,"master",i2.2,a)') trim(lmaster_dir),iyear2,lsuffix 
!    write(*,*) trim(lmasterfile)
     inquire(file=lmasterfile, exist=kexist)
     if(.not. kexist) cycle
     kopen(itype)=.true. 
     write(*,*) "Reading "//trim(lmasterfile)
     open(lu_master(itype),file=lmasterfile)
     read(lu_master(itype),'(a)',end=55) ldum 
     goto 60  
! come here if problems reading first lines       
55   continue
     kopen(itype)=.false.
     close(lu_master(itype))
     cycle 
     
60   continue           
! Data lines start with "|"  Space until we get there.         
     do while(ldum(1:1) .ne. "|")        
       read(lu_master(itype),'(a)',end=55) ldum
     end do
     if(kdebug) write(*,*) trim(ldum) 
!Data lines consist of fields separted by "|". Find out where these are. Within each file they are consistent.   
!     write(*,*) ldum(1:50) 
! Found a line with a session in it. Fields are separated by "|" Find where these are.  
     num_bar(itype)=0
     do j=1,len_trim(ldum)  
      if(ldum(j:j) .eq. "|") then
        num_bar(itype)=num_bar(itype)+1
        ibar_vec(itype,num_bar(itype))=j
       endif
     end do  
! Read in the first session     
!     writE(*,*) "bar: ", ibar_vec(itype,1:13) 
     call read_master_line(ldum,session_tmp(itype),ibar_vec(itype,:))      
     session_tmp(itype)%mjday=session_tmp(itype)%idoy+mjday_off
!     write(*,*) itype, session_tmp(itype)      
   end do  
   
! At this point have readd in a session from all of the master files that exist for this year. 
! Find the session type which is earliest and insert it. Read in another session from that master file.
! Continue until we are done. 
100  continue   
   do while(.true.)       
      itype=0    
      if(kopen(1)) then
         itype=1
      endif
      if(kopen(2)) then
         if(itype .eq. 0) then        
           itype=2
         else
           if(kge_session(session_tmp(itype),session_tmp(2))) itype =2                
          endif 
      endif
      if(kopen(3)) then 
         if(itype .eq. 0) then
            itype=3
         else
            if(kge_session(session_tmp(itype),session_tmp(3))) itype =3    
         endif 
      endif
      if(itype .eq. 0) exit         !This means we have no more sessions. 
      call insert_near_end(session_tmp(itype),session_list,max_session,num_session)
      ldum(1:1) = " "
! Find next line that has a session and read it in.       
      do while(ldum(1:1) .ne. "|")        
         read(lu_master(itype),'(a)',end=150) ldum
      end do
      call read_master_line(ldum,session_tmp(itype),ibar_vec(itype,:))      
      session_tmp(itype)%mjday=session_tmp(itype)%idoy+mjday_off
!      write(*,*) itype, session_tmp(itype)
      cycle
150   continue
      close(lu_master(itype))
      kopen(itype) = .false.             
   end do    
end do

110 continue 
!write(*,*) num_normal,num_int,num_vgos

if(.false.) then 
  open(11,file="temp.txt")
  do i=1,num_session
    write(11,*) i, session_list(i) 
  end do 
  close(11) 
!  stop 
endif 
return

900 continue
write(*,*) "Should never get here!" 

end subroutine 
!*********************************************************************   
subroutine read_master_line(ldum,session_tmp,ibar_vec)
implicit none
character*(*) ldum
integer  mjday_year_beg
type(session) session_tmp
integer ibar_vec(*)
! local
integer idoy

! put the idoy in this slot. 
read(ldum(ibar_vec(4)+1:ibar_vec(5)-1),*)  session_tmp%idoy

if(ldum(ibar_vec(12)+1:ibar_vec(13)-1) .eq. " ") then
   session_tmp%ldbc=" "
else     
   read(ldum(ibar_vec(12)+1:ibar_vec(13)-1),*) session_tmp%ldbc
endif

read(ldum(ibar_vec(2)+1:ibar_Vec(3)-1),*)   session_tmp%lsessionID              
end subroutine

!*******************************************************************
subroutine insert_near_end(session_in,session_list,max_session,num_session)
implicit none
type(session) :: session_in             !new session
type(session) :: session_list(*)        !
integer num_session                     !number of sessions currently in list.
integer max_session                     !maximum number of sessions. 

! local 
integer iput                            !where to put the  new session. 
type(session) :: session_tmp            !used locally to hold copy

! Insert session_in into session_list.  
! Assume that session_list is ordered by  mjday and ldbc 
! Want to make sure that we insert session_tmp in the correct spot.
! Issue is that we can have several sessions on the same day. Want to be sure that when we insert a new one that the ldbc are in the correct order.

if(num_session .eq. 0) then 
  num_session=num_session+1
  session_list(num_session)=session_in
  return
endif

if(num_session .eq. max_session) then
  write(*,*) "Insert_session_near_end:  Out of space!"
  write(*,*) "num_session ", num_session
  stop
endif 

num_session=num_session+1 

! In this loop we start at the end and stop when session_in is .GE. session_list(iput)
! Until we reach this space we keep on moving elements down a level to leave space. 
iput=num_session

do while(.not. kge_session(session_in, session_list(iput-1))) 
! Write need to make  space.    
  session_list(iput)=session_list(iput-1) 
  iput=iput-1
  if(iput .eq. 0) then
    write(*,*) "Help, should never get here"
    stop
   endif
end do 
session_list(iput)=session_in 

return 
end subroutine 

!**********************************
character*8 function get_sessionID(ldbname)
use time_stuff 
implicit none 
! Return the session ID from the master files based on the dbname
character*10 ldbname


!variables associated with session 
character*36 lmonths/"JANFEBMARAPRMAYJUNJULAUGSEPOCTNOVDEC"/   
integer iyear,imonth,iday
integer iyear2                      !Two digit year. 
integer mjday 
character*2 ldbc                     !XA, XU, XI etc

! miscellaneous
integer ind
integer i
logical kdebug
integer ilo, ihi,itest
type(session) session_tmp

kdebug=.false. 

! Find the database DOY and ldbc

! The database name is something like:
! $10JAN04XA 
! Extract the year, month date from database name
read(ldbname(2:3),*) iyear2
if(iyear2 .ge. 79) then
  iyear=iyear2+1900
else
  iyear=iyear2+2000
endif 

ind=index(lmonths,ldbname(4:6))
if(ind .eq. 0) then
   write(*,*) "Not a valid month ", ldbname(4:8)
   write(*,*) "ldbname "//ldbname
   stop
endif
imonth=1+(ind-1)/3
read(ldbname(7:8),*) iday
mjday=ymd_2_mjday(iyear,imonth,iday) 
ldbc=ldbname(9:10)

! Make a fake session
session_tmp%ldbc=ldbc
session_tmp%mjday=mjday
session_tmp%lsessionID=" "
session_tmp%idoy=0 

if(.false.) then 
!Below is brute force way of doing things, but it works. 
! First try normal sessions. 
if(kdebug) write(*,*) "Trying Normal" 
do i=1,num_session
   if(session_list(i)%mjday .eq. mjday .and. session_list(i)%ldbc .eq. ldbc) then  
      get_sessionID=Session_list(i)%lsessionID
      return
   endif
end do
endif 

100 continue 
! Here we use bisection to speed up the search
ilo=1
ihi=num_session
if(kdebug) write(*,*) "Finding : ", session_tmp 

do while(ihi-ilo .ge. 4) 
  itest=(ihi+ilo)/2
  if(kdebug) then
     write(*,*) "lo  hi test ",ilo,ihi,itest
     write(*,*)  session_list(itest)
  endif 
  if(kge_session(session_list(itest), session_tmp)) then 
    ihi=itest
  else
    ilo=itest
  endif 
end do

!when the list is short enough just check everything.
do itest=ilo,ihi
  if(session_tmp%mjday .eq. session_list(itest)%mjday .and. &
&     session_tmp%ldbc .eq. session_list(itest)%ldbc) then
!     write(*,*) "Found it" 
     get_sessionID=session_list(itest)%lsessionID
!     write(*,*) "Found it ", get_SessionID
!     read(*,*) 
      return 
  endif
end do 


write(*,*) "Did not find database "//ldbname
get_sessionID="UNKNOWN"
return
end function 
! ***************************************************************************************
   
end module master_stuff
