subroutine parse_control_file(lControlFile,ierr)
! Read in control file
! This contains info about directories, input and output files.
! 

use common_variables
use header_stuff 
use eop_stuff

implicit none 
character*(*) lControlFile
integer ierr

!local
logical kexist
integer, parameter :: lu_in=15

inquire(Exist=kexist, file=lControlFile)
if(.not. kexist) then
  write(*,*) "Did not find control file "//trim(lControlFile)
  stop
endif

lcontact="NONE"
ldescription="NONE"

leap_second_file=" " 

lcrf_apriori_override=" "
ltrf_apriori_override=" "
leop_apriori_override=" "
ldescription_override=" " 
leop_subdaily_override=" " 
lagency_override=" " 

! Open and parse the control file. It is of the form
! KEYWORD  VALUE
! KEYWORD VALUE
! lines that begin with "!" are comments and are ignored.

lnutation_type="CIO_BASED"       !This is the default
eop_sig_max=0.d0                 !0 means no maximum value. Non-zero means don't write out eop line if larger than this. 

open(lu_in,file=lcontrolFile)

do while(.true.) 
  read(lu_in,'(a)',end=500)  ldum
  if(ldum(1:1) .eq. "!") cycle       !ignore comments 
  if(ldum .eq. " " )  cycle          !ignore blank lines 
  call splitntokens(ldum,ltoken,Max_Token,Num_Token) 
  call capitalize(ltoken(1))
  if(Num_token .lt. 2) then 
    write(*,*) "Error parsing control file. Missing argument for keyword "//trim(ltoken(1))
    cycle
  endif 
  ind=index(ldum,trim(ltoken(2)))    !Find start of second token       Several options below need this data. 
  
  select case(ltoken(1)) 
    case("MASTER_DIR")
       lmaster_dir=ltoken(2)
    case("SPOOLFILE")
       lspoolfile=ltoken(2)
    case("EOP_FILE")
       leop_file=ltoken(2)
    case("NSCODES_FILE") 
       lnscodes_file=ltoken(2) 
    case("LEAP_SECOND_FILE")
       leap_second_file=ltoken(2)           
    case("CONTACT")    
       lcontact=ldum(ind:) 
    case("DESCRIPTION") 
       ldescription_override=ldum(ind:)     
    case("ANALYSIS_CENTER") 
       lanalysis_center=ldum(ind:)     
    case("AGENCY")
       lagency_override=ltoken(2)               
    case("CRF_APRIORI")
       lcrf_apriori_override=ltoken(2)
     case("TRF_APRIORI")
       ltrf_apriori_override=ltoken(2)
     case("EOP_APRIORI")
       leop_apriori_override=ltoken(2)
     case("EOP_SUBDAILY")
       leop_subdaily_override=ltoken(2)     
     case("XWOB_SIG_MAX","XPOL_SIG_MAX")
        reaD(ltoken(2),*,err=900)  eop_sig_max(1)
     case("YWOB_SIG_MAX","YPOL_SIG_MAX")    
        reaD(ltoken(2),*,err=900) eop_sig_max(2)
     case("UT1_SIG_MAX")
       reaD(ltoken(2),*,err=900)  eop_sig_max(3)
     case("NUT_DPSI_SIG_MAX","NUT_DX_SIG_MAX") 
        reaD(ltoken(2),*,err=900)  eop_sig_max(4) 
     case("NUT_DEPS_SIG_MAX","NUT_DY_SIG_MAX")
        read(ltoken(2),*,err=900) eop_sig_max(5)              
       
     case("NUTATION_TYPE")
       if(ltoken(2) .ne. "EQUINOX-BASED" .and. ltoken(2) .ne. "CIO-BASED") then
          write(*,*) "Invalid NUTATION_TYPE: "//trim(ltoken(2)) 
          write(*,*) "  valid options are EQUINOX-BASED or CIO-BASED!"
          write(*,*)
          stop
       endif       
       lnutation_type=ltoken(2)  
     case default
       write(*,*) "Unknown keyword "//trim(ltoken(1))
  end select
end do  

500 continue

! Do some checking.
if(lmaster_dir .eq. " ") then
  writE(*,*) "Must specify master_dir"
  stop
endif

if(leap_second_file .eq. " ") then
  write(*,*) "Must specify leap_second_file"
  stop
endif 

close(lu_in) 
return

900 continue
writE(*,*) "Error parsing >"//trim(ldum)
close(lu_in) 

stop 
return
end 


