program spool2eop
! Program to read a spool file, extract eop, and output in new eop format.
!use eop_data
use header_stuff 
use eop_stuff
use common_variables
use time_stuff 
implicit none

! History
! When changes are made update lversion in common_variables.f90
! 2022-06-10   First version.

! Temporary files 
character*32 leop_tmp
character*32 leop_sort_tmp  

integer,parameter :: lu_spool=25                         !handle for spool file
integer,parameter :: lu_tmp  =26                         !handle for temporary files. 
integer ierr                                              !an error
integer ipid

integer time_values(8)
Integer NumArg
Character*128 lControlFile 

! General description
! We read the spool file, extracting information, and putting it in the eop file.
! The eop file contains
!   1. Header information.
!   2. EOP information.
! When we read the spool file some of the header information occurs near the top, and some near the very end.
! In addition, there is no guarantee that the sessions in the spool file are in time order. 
! 
! Hence in reading the EOP file we do the following.
! 1. Extract header information from near the top of the spoolfile and store it internally.
! 2A Extract EOP information on a session-by-session-basis and write it out to a scratch file EOP_PID.txt 
! 2B Use a system utility to sort the file and create EOP_sort_PID.txt 
! 3. Extract the remaining header information from the end of the spool file. 
! 4A. Create the EOP file.
! 4B. Write out the header information.
! 4C. Write out the EOP information. (Done by copying lines from the scratch file.)
! 4D. Close EOP.
! 5.  Cleanup 

! Start of code.

write(*,*) lversion 

if(.true.) then  
  NumArg=iargc()
  if(NumArg .eq. 0) then
    write(*,*) "Syntax is: "
    write(*,*) "  spool2eop control_file"
    stop
  endif
  call getarg(1,lControlFile)
else
! hardwired for debugging
  lcontrolFile="/Fortran Code/spool2eop/example/spool2eop_example1_int.cnt"
!   lcontrolFile="/Fortran Code/spool2eop/example/spool2eop_example1.cnt"
endif 

call parse_control_file(lControlFile,ierr)

write(*,*) "Reading "//trim(lspoolfile)
write(*,*) "Creating "//trim(leop_file)

! Get the time when the program started. 

call date_and_time(VALUES=time_values)
! write(*,*) time_values
iutc_prog(1:3) = time_values(1:3)
iutc_prog(4:6) = time_values(5:7)


! Create names of temporary files.  This assumes there is /tmp directory. 
! Use PID number to make sure they are unique. 
ipid=getpid()
write(leop_tmp, '("/tmp/EOP_", i10.10,".txt")') ipid
write(leop_sort_tmp,'("/tmp/EOP_sort", i10.10,".txt")') ipid

open(lu_spool, file=lspoolfile)
open(lu_tmp,  file=leop_tmp) 

! Read the spool file and extract the information we need for the header file. 
!1. First read some stuff at the very top. Info like version number, when run done, etc. 
call parse_spool_header(lu_spool,ierr) 
write(*,*) "After header ", ierr

! Read in the EOP data from the spool file, and write it to a temporary file. Close the file when done. 
call parse_spool_eop_data(lu_spool,lu_tmp,ierr)
if(ierr .ne. 0) then
  write(*,*) "Problem in parsing EOP data"
  stop
endif 

! This sorted version of the file. 
call execute_command_line("sort <"//trim(leop_tmp)//">"//trim(leop_sort_tmp))
open(lu_tmp,  file=leop_tmp) 
! close(lu_tmp,status="Delete")   !Delete the temporary file. Note the sorted version still exists. 

! Now process the control section of the spool file. This has additional information we need for the header file.
call parse_spool_control_section(lu_spool,ierr) 

! We are now done the hard work and gathered all of the information we need. 
! Write the file out. 

call write_eop_file(leop_sort_tmp,leop_file) 

write(*,*) "Done"
end program



