module time_stuff
!subroutines dealing with time...

contains 
!************************************************
subroutine round_trip_check()
! Check the results roundtrip
implicit none 
integer iyear, imonth, iday,mjday
double precision rmjd

do while(.true.)
  write(*,'("Enter in Year, month, day: ",$)') 
  read(*,*) iyear,imonth, iday 
  if(iyear .eq. -1) return 
  mjday=ymd_2_mjday(iyear,imonth,iday)
  write(*,*) "MJDAY: ", mjday
  rmjd=mjday+1e-3     !Add a bit to make sure we don't round down. 
  call rmjd_2_ymd(rmjd,iyear,imonth,iday)
  write(*,*) "After round trip: ", iyear, imonth, iday 
end do 
end subroutine 

!******************************************************************
! Return Julian day at noon. 
!   * [USNO](http://aa.usno.navy.mil/faq/docs/JD_Formula.php)
pure integer function julian_day(y,m,d)
implicit none
integer,intent(in) :: y   !! year (YYYY)
integer,intent(in) :: m   !! month (MM)
integer,intent(in) :: d   !! day (DD)

julian_day = d-32075+1461*(y+4800+(m-14)/12)/4+367*(m-2-(m-14)/12*12)/12-3*((y+4900+(m-14)/12)/100)/4
end function julian_day

!*********************************************************
integer function ymd_2_mjday(y,m,d) 
implicit none
integer,intent(in) :: y   !! year (YYYY)
integer,intent(in) :: m   !! month (MM)
integer,intent(in) :: d   !! day (DD)

ymd_2_mjday=julian_day(y,m,d)-2400001
return
end 
    
!****************************************    

character*19 function lutc_2_iso_time(iutc)
implicit none 
integer iutc(6)     !y,m,d, h,m,s 
! Return time as ISO string

write(lutc_2_iso_time,'(i4.4,2("-",i2.2),"T",2(i2.2,":"),i2.2)') iutc
return
end function 

!***************************************************************
subroutine rmjd_2_utc(rmjd, iutc)
implicit none
double precision rmjd
integer iutc(6)

!local
double precision frac
integer iyear,imonth,iday,ihr,imin,isec

! Get the year,month,day part
call rmjd_2_ymd(rmjd,iyear,imonth,iday)

! now get the hour minutes, second
frac=(rmjd-int(rmjd))*86400.d0    !convert to seconds. 

ihr=int(frac/3600.d0)
frac=frac-ihr*3600.d0
imin=frac/60.d0
frac=frac-imin*60.d0
isec=frac 

! Fix up some edge cases
if(isec .eq. 60) then
  isec=0
  imin=imin+1
endif 

if(imin .eq. 60) then 
 imin=0
 ihr=ihr+1
endif 

iutc(1)=iyear
iutc(2)=imonth
iutc(3)=iday
iutc(4)=ihr
iutc(5)=imin
iutc(6)=isec 

return
end subroutine rmjd_2_utc

!********************************************************************
      SUBROUTINE rmjd_2_ymd (rmjd, YEAR,MONTH,DAY)
      implicit none  !2020Jun15 JMGipson automatically inserted.

! modified version of a program from USNO. Original program used JD. This uses rMJD as a real  
! aa.usno.navy.mil/faq/docs/JD_formula.php
! Copied by JMGipson on 2019.06.06
!
!---COMPUTES THE GREGORIAN CALENDAR DATE (YEAR,MONTH,DAY)
!   GIVEN THE JULIAN DATE (JD).
      double precision rmjd  !input 
      integer YEAR,MONTH,DAY
     
! internal     
      INTEGER*4 JD
      integer*4 I,J,K, L,N      
      jd=nint(rmjd+2400000.5d0)
 !
      L= JD+68569
      N= 4*L/146097
      L= L-(146097*N+3)/4
      I= 4000*(L+1)/1461001
      L= L-1461*I/4+31
      J= 80*L/2447
      K= L-2447*J/80
      L= J/11
      J= J+2-12*L
      I= 100*(N-49)+I+L

      YEAR= I
      MONTH= J
      DAY= K

      RETURN
      END subroutine
! **************************************************************************      

end module time_stuff 